{{******************************************************
* (C) 2009 myDanceBot.com, Inc.                        *
* Conduit v4.2 6/09                                    *
* Share Propeller Memory with ViewPort Application     *
********************************************************
Use this Object to share memory with a ViewPort Application.
Once memory is shared, the value of a variable can be viewed and changed while the
program is running.  Data from other objects can be displayed and their behavior
controlled from ViewPort's graphical interface.
_______________________
Features:
*Transfers data at up to 2Mbps Full-Duplex over a Parallax USB-Serial connection or
 115kbps over other Serial connections.
*Manages configuration of variables and registration of ViewPort objects
*Automatically switches between 2Mbps, 1Mbps and 115kbps depending on ViewPort setting
_______________________
Use:
-Set clock rate to 80MHz:
        CON
         _clkmode        = xtal1 + pll16x
         _xinfreq        = 5_000_000
-Include this object in your program:
        Obj vp:"Conduit"
-Use these ViewPort Functions
  Share(@varA,@varC)            Share memory from varA to varC with ViewPort. This lets you
                                monitor&change them.  You MUST use this function- at the start of your
                                program, but AFTER "register" and "config" statements have run.
  Register(component,cfg)       Register other components (fuzzy logic, video, quicksample..)
  Config(cfgstring)             Configure ViewPort.  Can be generated by ViewPort.
}}
CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  TXPIN=|<30  
  RXPIN=|<31
  PS=True
  DBGLEN=6
PUB debug(dP1,dP2)
{{
 Used by ViewPort debugger
}}
  dbgPtr:=dP1 
  dbPtr:=dP2  
PUB share(firstVarPtr,lastVarPtr)
{{
Share memory from varA to varC with ViewPort. This lets you
monitor&change them.  You MUST use this function- at the start of your
program, but AFTER "register" and "config" statements have run.  
}}
  if lastVarPtr
    varLen:=((lastVarPtr-firstVarPtr+4)/4)& $fff

    varPtr:=firstVarPtr
    txvar~
  resyncP:=1200/4/(varlen+tvlen+qslen+fzlen)
 
  if resyncP<2
    resyncP:=1
  resyncCnt:=resyncP
  ''calculate timings
  bittick10:=clkfreq/2_000_000            
  bittick15:=bittick10*3/2               
  bitticks115k:=clkfreq/115_200
  packetNPtr:= @packetN
  cognew(@entry, varPtr)
  return @packetN
pub config(sptr)
{{
Configure ViewPort.  CfgString can be generated by ViewPort.
}} 
  if not txstrPtr0
    txStrPtr0:=sptr
  elseif not txstrPtr1
    txStrPtr1:=sptr
  elseif not txstrPtr2
    txStrPtr2:=sptr
  elseif not txstrPtr3
    txStrPtr3:=sptr
  elseif not txstrPtr4
    txStrPtr4:=sptr
  elseif not txstrPtr5
    txStrPtr5:=sptr
pub register(mPtr)|mode,cfg
{{
Register other components (fuzzy logic, video, quicksample..)
}}
   
   
   
   
  mode:=mptr & 3
  mPtr-=mode
  cfg:=long[mPtr]
  case mode
    0:
      case cfg
        1..2:tvLen:=6000
        3:tvLen:=6000+1
        4:tvLen:=3000+1
        OTHER: tvLen:=1500
      tvPtr:=mptr
      txtv~
    1:'     (config>>16)==0




       memPtr:=mPtr+8
    2: 
   


      qsPtr:=mptr              
      qscog:=cfg               
      tcog:=long[mPtr+8]
      ncog:=long[mPtr+12]+1    
      qsLen:=ncog*360+6                    
      if ncog==5
        ncog-=1
        qslen:=4*258+6     
        psadd2+=1        
        sync:=mptr+24
        fPacketPtr:=mPtr+28   
      else


      
        psinitT:=0     
        sync:=mPtr+24
        fpacketPtr:=mPtr+28

      txqs~
    3:
   
      fzPtr:=mPtr
      fzLen:=cfg/4
      txfz~
DAT
{************************************
* Assembly language conduit driver  *
*************************************}

                        org
entry
                        sub     tximask,#1
                        sub     tximask,txmask
                        mov     txcnt,hsec
                        add     txcnt,cnt
                       
                        waitcnt txcnt,#100
                        or      dira,txmask
                        or      outa,txmask
                        jmp     #txcfgstr

receive                 or      outa,txmask
                        mov     txcnt,cnt 
                        waitpeq rxmask,rxmask
                        mov     t1,cnt
                        sub     txcnt,t1
                        mov     t4,#2 
                        mov     t5,#0 
                        neg     txcnt,txcnt 
                        cmp     txcnt,stopticks wc 
            if_c        jmp     #txErr
                        waitpne rxmask,rxmask

                        mov     rxcnt,cnt
                        mov     t3,rxcnt't2-t3
                        sub     t3,t1  
                       
{t2 measures stopbit length.
  if t2<80, 80mhz/2mbit=40cycles
  stopbits is 2x bitticks
  if you're running at a different frequency, change bitticks, stopticks and xtf
  for example, if you're running at 96mhz, bitticks should be=48, stopticks=96}
                        cmp     t3,bittick15 wc    
            if_c        mov     bitticks,bittick10
            if_c        jmp     #bit0
                        mov     t1,bittick15
                        shl     t1,#1 
                        cmp     t3,t1 wc    
            if_c        mov     bitticks,bittick10
            if_c        shl     bitticks,#1 
            if_c        jmp     #bit0                         
                        mov     bitticks,bitticks115k
                        mov     t1,bitticks
                        shr     t1,#1
                        add     rxcnt,t1
bit0
                        mov     stopticks,bitticks
                        shl     stopticks,#1
                        add     rxcnt,bitticks
                        add     rxcnt,#12
bit                    
                        mov     t2,#8
                       
                       
  :bit8                 waitcnt rxcnt,bitticks
                        test    rxmask,ina      wc
                        rcr     t5,#1    
                        djnz    t2,#:bit8
                        waitcnt rxcnt,stopticks
                        djnz    t4,#bit
                        shr     t5,#16   
                        mov     t4,#4
 rxlong
                        mov     t2,#8
                       
  :bit8                 waitcnt rxcnt,bitticks
                        test    rxmask,ina      wc
                        rcr     t1,#1    
                        djnz    t2,#:bit8
                        waitcnt rxcnt,stopticks
                        djnz    t4,#rxlong    
                        cmp     t5,#0 wz
              if_z      add     t1,#:docontrol
              if_z      jmp     t1

                        wrlong  t1,t5   
                        jmp     #txData

:docontrol              jmp     #txErr   
                        jmp     #txMem   
                        jmp     #txData  
                        jmp     #restart 
                        jmp     #txcfgStr
                        jmp     #:leavebkpt'5: leave bkpt
                        jmp     #txAll    
:leavebkpt
                        mov     t2,dbPtr
                        add     t2,#4     
                        rdlong  t3,t2     
                        mov     t1,#0
                        wrlong  t1,t2     
                        mov     rxcnt,cnt
                        add     rxcnt,spinticks
                        waitcnt rxcnt,bitticks 
                        wrlong  t3,t2     
                        wrlong  t1,dbPtr  
                        jmp     #txData
  restart               mov     t1,cnt
                        wrlong  t1,sync
                        wrlong  packetNPtr,fPacketPtr
                        mov     t1,qscog   
                        mov     t2,ncog
  startC                coginit t1 nr
   psadd2               add     t1,#1   
                        djnz    t2,#startC
   psinitT              coginit tcog    
txData
                        mov     conduitstate,conduitstate2
                        mov     txcnt,cnt
                        add     txcnt,stopticks
                        mov     rxcnt,#1
                        call    #txhdr
txD                     mov     DivCnt,#256                        
                       
txMultB                 mov     doIncStatus,#1
         txvar          jmp     #txqs
                        mov     t2,varLen
                        mov     t3,varPtr
                        call    #txlongs
         txqs           jmp     #txtv                              
                        mov     t2,qsLen
                        mov     t3,qsPtr
                        call    #txlongs
         txtv           jmp     #txfz                              
                        mov     t2,tvLen
                        mov     t3,tvPtr
                        call    #txlongs                        
         txfz           jmp     #txDDone
                        mov     t2,fzLen
                        mov     t3,fzPtr
                        call    #txlongs
                        
txDDone                        
                      
                      
txDataCon
'                        jmp     #txMultB                       
                        djnz    resynccnt,#txMultB
                        mov     resynccnt,resyncP

                      
                        mov     t2,#6'dbgLen
                        mov     t3,dbgPtr
                        call    #txlongs
                        call    #transmit
                                                  
 
                        mov     rxcnt,#199
                        neg     rxcnt,rxcnt
                        call    #txhdr   
                        jmp     #txD
                    
txlongs                 call    #transmit
                        djnz    divCnt,#notEOD
                       
                       
                       
                        mov     txByte,#26
                        call    #transmit  
                        mov     divCnt,#255
      notEOD            rdlong  txByte,t3
                        call    #transmit
                        cmp     txByte,s1 wz
                        add     t3,#4                                
                        call    #transmit
                        tjz     doIncStatus,#:dontIncStatus
                        add     packetN,#1
                        wrlong  packetN,packetNPtr
                        mov     doIncStatus,#0
:dontincstatus          call    #transmit
              if_z      call    #translong

              if_z      add     txByte,#6                        
                        djnz    t2,#txlongs
                        
txlongs_ret             ret

                      
txErr                   mov     txcnt,cnt
                        add     txcnt,stopticks
                        mov     rxcnt,#5
                        call    #txhdr
                        jmp     #txcfgstr
                        
txhdr                   mov     txByte,s1
                        call    #translong
                        add     txByte,rxcnt
                        ror     txByte,#8
                        call    #translong
txhdr_ret               ret
             

txMem                   mov     txcnt,cnt
                        add     txcnt,stopticks
    :txloop             mov     t2,memPtr
                        rdlong  t3,t2
                        call    #translong
                        mov     rxcnt,#3
                        call    #txhdr
                        jmp     #:txloop
'sends out memory, header
txAll                   mov     t3,#0
                        mov     rxcnt,#8                        
                        mov     txcnt,cnt
                        add     txcnt,stopticks
                        call    #txhdr
                        call    #transmit
                        mov     t4,#32                     
    :txloop            
                       
                       
                       
                        mov     t2,#256


     :txlongs           call    #transmit
                        rdlong  txByte,t3
                        call    #transmit
                        cmp     txByte,s1 wz
                        add     t3,#4                                
                        call    #transmit
                        call    #transmit
              if_z      call    #translong
              if_z      add     txByte,#6                        
                        djnz    t2,#:txlongs
                        



                        mov     txByte,#26
                        call    #transmit
                        djnz    t4,#:txloop

                        mov     rxcnt,#7
                        call    #txhdr
                        jmp     #txdata                                                                       

txCfgStr                mov     txcnt,cnt
                        add     txcnt,stopticks                        
    :txloop             mov     txByte,varLen       
                        add     txByte,conduitState
                        call    #transLong
                        mov     txByte,qsLen        
                        call    #transLong
                        mov     txByte,tvLen        
                        call    #transLong
                        mov     txByte,fzLen        
                        call    #transLong
                        mov     txByte,memPtr       
                        call    #transLong
                        mov     txByte,varPtr       
                        call    #transLong
                        mov     txByte,qsPtr        
                        call    #transLong
                        mov     txByte,fzPtr        
                        call    #transLong
                        mov     txByte,dbPtr        
                        call    #transLong
                        rdlong  txByte,#0           
                        call    #translong                     
                        mov     t2,txstrptr0
                        call    #txStr
                        mov     t2,txstrptr1
                        call    #txStr
                        mov     t2,txstrptr2
                        call    #txStr
                        mov     t2,txstrptr3
                        call    #txStr
                        mov     t2,txstrptr4
                        call    #txStr
                        mov     t2,txstrptr5
                        call    #txStr
                        mov     rxcnt,#4
                        call    #txhdr
                        jmp     #:txloop                        
txStr                   add     txcnt,bitticks
                        tjnz    t2,#:chars
                        mov     txByte,#0
                        call    #transmit
                        jmp     #txStr_ret
:chars                  rdbyte  txByte,t2
                        mov     t3,txByte
                        call    #transmit
                        add     t2,#1
                        tjnz    t3,#:chars
txStr_ret               ret                        

transLong
                        call    #transmit
                        call    #transmit
                        call    #transmit
                        call    #transmit
transLong_ret           ret

transmit                waitcnt txcnt,bitticks   
                        and     outa,tximask    
                        mov     t1,#8
:bit                    test    rxmask,ina      wc                        
            if_nc       jmp     #receive
                        ror     txByte,#1       wc
                        waitcnt txcnt,bitticks     
                        muxc    outa,txmask
                        djnz    t1,#:bit
                        waitcnt txcnt,stopticks                                          
                        or      outa,txmask     
transmit_ret            ret
bitticks                long    40
stopticks               long    80
s1                      long    $E1E1E1E1
txstrptr0               long    0
txstrptr1               long    0
txstrptr2               long    0
txstrptr3               long    0
txstrptr4               long    0
txstrptr5               long    0

varPtr                  long    0
qsPtr                   long    0
tvPtr                   long    0
fzPtr                   long    0
memPtr                  long    0

varLen                  long    0
qsLen                   long    0
tvLen                   long    0
fzLen                   long    0
conduitState            long    104<<16
conduitState2           long    105<<16



hsec                    long    8_000_000
resyncp                 long    0 
resynccnt               long    0 


qscog                   long    0
tcog                    long    0
sync                    long    0
ncog                    long    0

t1                      long    0 
t2                      long    0
t3                      long    0
t4                      long    0
t5                      long    0
rxmask                  long    RXPIN
txmask                  long    TXPIN
tximask                 long    0
spinticks               long    2000
dbptr                   long    1
dbgPtr                  long    1
{constants for slow serial connection- at 80mhz equates to 115kbps.
xt is the time in cycles to send 360 longs, at 96mhz should be 17_280_000}
bittick10               long    0
bittick15               long    0
bitticks115k            long    0
packetNPtr              long    0
packetN                 long    0
fPacketPtr              long    0

xt                      res     1
rxcnt                   res     1
txByte                  res     1
txcnt                   res     1
doIncStatus             res     1
divcnt                  res     1